@@ -10,7 +10,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-SKIP_WITH_LVMLOCKD=1
+
 
 . lib/inittest
 
@@ -36,15 +36,6 @@ not lvconvert -m2 $vg/lv
 # Log conversion (disk -> core)
 lvconvert --mirrorlog core $vg/lv
 
-# Log conversion (core -> mirrored)
-# FIXME on cluster
-SHOULD=""
-test -e LOCAL_CLVMD && SHOULD=should
-$SHOULD lvconvert --mirrorlog mirrored $vg/lv
-
-# Log conversion (mirrored -> core)
-lvconvert --mirrorlog core $vg/lv
-
 # Log conversion (core -> disk)
 lvconvert --mirrorlog disk $vg/lv
 
