@@ -15,10 +15,10 @@
 
 #include "tools.h"
 
-#include "polldaemon.h"
-#include "display.h"
+#include "lib/lvmpolld/polldaemon.h"
+#include "lib/display/display.h"
 #include "pvmove_poll.h"
-#include "lvmpolld-client.h"
+#include "lib/lvmpolld/lvmpolld-client.h"
 
 #define PVMOVE_FIRST_TIME   0x00000001      /* Called for first time */
 
@@ -67,10 +67,6 @@ static int _pvmove_target_present(struct cmd_context *cmd, int clustered)
 static unsigned _pvmove_is_exclusive(struct cmd_context *cmd,
 				     struct volume_group *vg)
 {
-	if (vg_is_clustered(vg))
-		if (!_pvmove_target_present(cmd, 1))
-			return 1;
-
 	return 0;
 }
 
@@ -155,6 +151,7 @@ static int _remove_sibling_pvs_from_trim_list(struct logical_volume *lv,
 					      struct dm_list *trim_list)
 {
 	char *idx, *suffix;
+	const char *sibling;
 	char sublv_name[NAME_LEN];
 	struct logical_volume *sublv;
 	struct dm_list untrim_list, *pvh1, *pvh2;
@@ -169,7 +166,16 @@ static int _remove_sibling_pvs_from_trim_list(struct logical_volume *lv,
 
 	dm_list_init(&untrim_list);
 
-	if (!(suffix = first_substring(lv_name, "_rimage_", "_rmeta_", NULL))) {
+	if (!dm_strncpy(sublv_name, lv_name, sizeof(sublv_name))) {
+		log_error(INTERNAL_ERROR "LV name %s is too long.", lv_name);
+		return 0;
+	}
+
+	if ((suffix = strstr(sublv_name, "_rimage_")))
+		sibling = "meta";
+	else if ((suffix = strstr(sublv_name, "_rmeta_")))
+		sibling = "image";
+	else {
 		log_error("Can't find rimage or rmeta suffix.");
 		return 0;
 	}
@@ -181,8 +187,8 @@ static int _remove_sibling_pvs_from_trim_list(struct logical_volume *lv,
 	idx++;
 
 	/* Create the siblings name (e.g. "raidlv_rmeta_N" -> "raidlv_rimage_N" */
-	if (dm_snprintf(sublv_name, sizeof(sublv_name), "%s_r%s_%s", lv_name,
-			strstr(suffix, "_rimage_") ? "meta" : "image", idx) < 0) {
+	if (dm_snprintf(suffix + 2, sizeof(sublv_name) - 2 - (suffix - sublv_name),
+			"%s_%s", sibling, idx) < 0) {
 		log_error("Raid sublv for name %s too long.", lv_name);
 		return 0;
 	}
@@ -313,32 +319,6 @@ static int _sub_lv_of(struct logical_volume *lv, const char *lv_name)
 	return _sub_lv_of(seg->lv, lv_name);
 }
 
-/*
- * parent_lv_is_cache_type
- *
- * FIXME: This function can be removed when 'pvmove' is supported for
- *        cache types.
- *
- * If this LV is below a cache LV (at any depth), return 1.
- */
-static int _parent_lv_is_cache_type(struct logical_volume *lv)
-{
-	struct lv_segment *seg;
-
-	/* Sub-LVs only ever have one segment using them */
-	if (dm_list_size(&lv->segs_using_this_lv) != 1)
-		return 0;
-
-	if (!(seg = get_only_segment_using_this_lv(lv)))
-		return_0;
-
-	if (lv_is_cache_type(seg->lv))
-		return 1;
-
-	/* Continue up the tree */
-	return _parent_lv_is_cache_type(seg->lv);
-}
-
 /* Create new LV with mirror segments for the required copies */
 static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 						struct volume_group *vg,
@@ -356,8 +336,8 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 	uint32_t log_count = 0;
 	int lv_found = 0;
 	int lv_skipped = 0;
-	int lv_active_count = 0;
-	int lv_exclusive_count = 0;
+	int needs_exclusive = *exclusive;
+	const struct logical_volume *holder;
 
 	/* FIXME Cope with non-contiguous => splitting existing segments */
 	if (!(lv_mirr = lv_create_empty("pvmove%d", NULL,
@@ -398,41 +378,41 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 		if (lv_name && strcmp(lv->name, top_level_lv_name(vg, lv_name)))
 			continue;
 
-		/*
-		 * RAID, thin and snapshot-related LVs are not
-		 * processed in a cluster, so we don't have to
-		 * worry about avoiding certain PVs in that context.
-		 *
-		 * Allow clustered mirror, but not raid mirror.
-		 */
-		if (vg_is_clustered(vg) && !lv_is_mirror_type(lv))
-			continue;
-
 		if (!lv_is_on_pvs(lv, source_pvl))
 			continue;
 
 		if (lv_is_converting(lv) || lv_is_merging(lv)) {
-			log_error("Unable to pvmove when %s volumes are present.",
-				  lv_is_converting(lv) ?
-				  "converting" : "merging");
+			log_error("Unable to pvmove when %s volume %s is present.",
+				  lv_is_converting(lv) ? "converting" : "merging",
+				  display_lvname(lv));
 			return NULL;
 		}
 
-		if (seg_is_raid(first_seg(lv)) ||
-		    seg_is_mirrored(first_seg(lv))) {
+		seg = first_seg(lv);
+		if (!needs_exclusive) {
+			/* Presence of exclusive LV decides whether pvmove must be also exclusive */
+			if (!seg_only_exclusive(seg)) {
+				holder = lv_lock_holder(lv);
+				if (seg_only_exclusive(first_seg(holder)) || lv_is_origin(holder) || lv_is_cow(holder))
+					needs_exclusive = 1;
+			} else
+				needs_exclusive = 1;
+		}
+
+		if (seg_is_raid(seg) || seg_is_mirrored(seg)) {
 			dm_list_init(&trim_list);
 
 			if (!get_pv_list_for_lv(vg->cmd->mem, lv, &trim_list))
 				return_NULL;
 
 			/*
- 			 * Remove any PVs holding SubLV siblings to allow
- 			 * for collocation (e.g. *rmeta_0 -> *rimage_0).
- 			 *
- 			 * Callee checks for lv_name and valid raid segment type.
- 			 *
- 			 * FIXME: don't rely on namespace
- 			 */
+			 * Remove any PVs holding SubLV siblings to allow
+			 * for collocation (e.g. *rmeta_0 -> *rimage_0).
+			 *
+			 * Callee checks for lv_name and valid raid segment type.
+			 *
+			 * FIXME: don't rely on namespace
+			 */
 			if (!_remove_sibling_pvs_from_trim_list(lv, lv_name, &trim_list))
 				return_NULL;
 
@@ -458,82 +438,53 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 			lv_found = 1;
 		}
 
+		seg = first_seg(lv);
+
+		if (seg_is_cache(seg) || seg_is_cache_pool(seg) ||
+		    seg_is_mirrored(seg) || seg_is_raid(seg) ||
+		    seg_is_snapshot(seg) ||
+		    seg_is_thin(seg) || seg_is_thin_pool(seg))
+			continue; /* bottom-level LVs only... */
+
 		if (!lv_is_on_pvs(lv, source_pvl))
 			continue;
 
-		if (lv_is_cache_type(lv)) {
-			log_print_unless_silent("Skipping %s LV, %s.",
-						lv_is_cache(lv) ? "cache" :
-						lv_is_cache_pool(lv) ?
-						"cache-pool" : "cache-related",
-						lv->name);
-			lv_skipped = 1;
-			continue;
-		}
-
-		if (_parent_lv_is_cache_type(lv)) {
-			log_print_unless_silent("Skipping %s because a parent"
-						" is of cache type.", lv->name);
-			lv_skipped = 1;
-			continue;
-		}
-
-		/*
-		 * If the VG is clustered, we are unable to handle
-		 * snapshots, origins, thin types, RAID or mirror
-		 */
-		if ((vg_is_clustered(vg) || is_lockd_type(vg->lock_type)) &&
-		    (lv_is_origin(lv) || lv_is_cow(lv) ||
-		     lv_is_thin_type(lv) || lv_is_raid_type(lv))) {
-			log_print_unless_silent("Skipping %s LV %s.",
-						lv_is_origin(lv) ? "origin" :
-						lv_is_cow(lv) ?
-						"snapshot-related" :
-						lv_is_thin_volume(lv) ? "thin" :
-						lv_is_thin_pool(lv) ?
-						"thin-pool" :
-						lv_is_thin_type(lv) ?
-						"thin-related" :
-						seg_is_raid(first_seg(lv)) ?
-						"RAID" :
-						lv_is_raid_type(lv) ?
-						"RAID-related" : "",
-						lv->name);
-			lv_skipped = 1;
-			continue;
-		}
-
-		seg = first_seg(lv);
-		if (seg_is_raid(seg) || seg_is_mirrored(seg) ||
-		    lv_is_thin_volume(lv) || lv_is_thin_pool(lv)) {
-			/*
-			 * Pass over top-level LVs - they were handled.
-			 * Allow sub-LVs to proceed.
-			 */
-			continue;
-		}
-
 		if (lv_is_locked(lv)) {
 			lv_skipped = 1;
-			log_print_unless_silent("Skipping locked LV %s.", lv->name);
+			log_print_unless_silent("Skipping locked LV %s.", display_lvname(lv));
 			continue;
 		}
 
-		if (vg_is_clustered(vg) &&
-		    lv_is_active_exclusive_remotely(lv)) {
+		holder = lv_lock_holder(lv);
+
+		if (needs_exclusive) {
+			/* With exclusive pvmove skip LV when:
+			 *  - is active remotely
+			 *  - is not active locally and cannot be activated exclusively locally
+			 *
+			 * Note: lvm2 can proceed with exclusive pvmove for 'just' locally active LVs
+			 * in the case it's NOT active anywhere else, since LOCKED LVs cannot be
+			 * later activated by user.
+			 */
+			if ((!lv_is_active(holder) && !activate_lv(cmd, holder))) {
+				lv_skipped = 1;
+				log_print_unless_silent("Skipping LV %s which is not locally exclusive%s.",
+							display_lvname(lv),
+							/* Report missing cmirrord cases that matterd.
+							 * With exclusive LV types cmirrord would not help. */
+							(*exclusive &&
+							 !lv_is_origin(holder) &&
+							 !seg_only_exclusive(first_seg(holder))) ?
+							" and clustered mirror (cmirror) not detected" : "");
+				continue;
+			}
+		} else if (!activate_lv(cmd, holder)) {
 			lv_skipped = 1;
-			log_print_unless_silent("Skipping LV %s which is activated "
-						"exclusively on remote node.", lv->name);
+			log_print_unless_silent("Skipping LV %s which cannot be activated.",
+						display_lvname(lv));
 			continue;
 		}
 
-		if (vg_is_clustered(vg)) {
-			if (lv_is_active_exclusive_locally(lv))
-				lv_exclusive_count++;
-			else if (lv_is_active(lv))
-				lv_active_count++;
-		}
-
 		if (!_insert_pvmove_mirrors(cmd, lv_mirr, source_pvl, lv,
 					    *lvs_changed))
 			return_NULL;
@@ -555,27 +506,9 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 		return NULL;
 	}
 
-	if (vg_is_clustered(vg) && lv_active_count && *exclusive) {
-		log_error("Cannot move in clustered VG %s, "
-			  "clustered mirror (cmirror) not detected "
-			  "and LVs are activated non-exclusively.",
-			  vg->name);
-		return NULL;
-	}
-
-	if (vg_is_clustered(vg) && lv_exclusive_count) {
-		if (lv_active_count) {
-			log_error("Cannot move in clustered VG %s "
-				  "if some LVs are activated "
-				  "exclusively while others don't.",
-				  vg->name);
-			return NULL;
-		}
-		*exclusive = 1;
-	}
-
-	if (!lv_add_mirrors(cmd, lv_mirr, 1, 1, 0, 0, log_count,
-			    allocatable_pvs, alloc,
+	if (!lv_add_mirrors(cmd, lv_mirr, 1, 1, 0,
+			    get_default_region_size(cmd),
+			    log_count, allocatable_pvs, alloc,
 			    (arg_is_set(cmd, atomic_ARG)) ?
 			    MIRROR_BY_SEGMENTED_LV : MIRROR_BY_SEG)) {
 		log_error("Failed to convert pvmove LV to mirrored.");
@@ -587,6 +520,9 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 		return NULL;
 	}
 
+	if (needs_exclusive)
+		*exclusive = 1;
+
 	return lv_mirr;
 }
 
@@ -595,10 +531,7 @@ static int _activate_lv(struct cmd_context *cmd, struct logical_volume *lv_mirr,
 {
 	int r = 0;
 
-	if (exclusive || lv_is_active_exclusive(lv_mirr))
-		r = activate_lv_excl(cmd, lv_mirr);
-	else
-		r = activate_lv(cmd, lv_mirr);
+	r = activate_lv(cmd, lv_mirr);
 
 	if (!r)
 		stack;
@@ -610,63 +543,34 @@ static int _activate_lv(struct cmd_context *cmd, struct logical_volume *lv_mirr,
  * Called to set up initial pvmove LV only.
  * (Not called after first or any other section completes.)
  */
-static int _update_metadata(struct cmd_context *cmd, struct volume_group *vg,
-			    struct logical_volume *lv_mirr,
-			    struct dm_list *lvs_changed, unsigned exclusive)
+static int _update_metadata(struct logical_volume *lv_mirr,
+			    struct dm_list *lvs_changed,
+			    unsigned exclusive)
 {
-	int r = 0;
+	struct lv_list *lvl;
+	struct logical_volume *lv = lv_mirr;
 
-	log_verbose("Setting up pvmove in on-disk volume group metadata.");
-	if (!vg_write(vg)) {
-		log_error("ABORTING: Volume group metadata update failed.");
-		return 0;
+	dm_list_iterate_items(lvl, lvs_changed) {
+		lv = lvl->lv;
+		break;
 	}
 
-	if (!suspend_lvs(cmd, lvs_changed, vg)) {
-		log_error("ABORTING: Temporary pvmove mirror activation failed.");
-		/* FIXME Add a recovery path for first time too. */
-		return 0;
-	}
+	if (!lv_update_and_reload(lv))
+                return_0;
 
-	/* Commit on-disk metadata */
-	if (!vg_commit(vg)) {
-		log_error("ABORTING: Volume group metadata update failed.");
-		if (!resume_lvs(cmd, lvs_changed))
-			log_error("Unable to resume logical volumes.");
-		return 0;
-	}
-
-	/* Activate the temporary mirror LV */
-	/* Only the first mirror segment gets activated as a mirror */
-	/* FIXME: Add option to use a log */
-	if (!exclusive && _pvmove_is_exclusive(cmd, vg))
-		exclusive = 1;
-
-	if (!_activate_lv(cmd, lv_mirr, exclusive)) {
-		if (test_mode()) {
-			r = 1;
-			goto out;
-		}
+	/* Ensure mirror LV is active */
+	if (!_activate_lv(lv_mirr->vg->cmd, lv_mirr, exclusive)) {
+		if (test_mode())
+			return 1;
 
 		/*
 		 * FIXME Run --abort internally here.
 		 */
 		log_error("ABORTING: Temporary pvmove mirror activation failed. Run pvmove --abort.");
-		goto out;
+		return 0;
 	}
 
-	r = 1;
-
-out:
-	if (!resume_lvs(cmd, lvs_changed)) {
-		log_error("Unable to resume logical volumes.");
-		r = 0;
-	}
-
-	if (r)
-		backup(vg);
-
-	return r;
+	return 1;
 }
 
 static int _copy_id_components(struct cmd_context *cmd,
@@ -696,6 +600,8 @@ static int _pvmove_setup_single(struct cmd_context *cmd,
 	struct dm_list *lvs_changed;
 	struct logical_volume *lv_mirr;
 	struct logical_volume *lv = NULL;
+	struct lv_list *lvl;
+	const struct logical_volume *lvh;
 	const char *pv_name = pv_dev_name(pv);
 	unsigned flags = PVMOVE_FIRST_TIME;
 	unsigned exclusive;
@@ -728,7 +634,7 @@ static int _pvmove_setup_single(struct cmd_context *cmd,
 	 * other hosts.  For LVs that are active on multiple hosts (sh), we
 	 * would need to used cluster mirrors.
 	 */
-	if (is_lockd_type(vg->lock_type)) {
+	if (vg_is_shared(vg)) {
 		if (!lv) {
 			log_error("pvmove in a shared VG requires a named LV.");
 			return ECMD_FAILED;
@@ -757,6 +663,13 @@ static int _pvmove_setup_single(struct cmd_context *cmd,
 			goto out;
 		}
 
+		dm_list_iterate_items(lvl, lvs_changed) {
+			lvh = lv_lock_holder(lvl->lv);
+			/* Exclusive LV decides whether pvmove must be also exclusive */
+			if (lv_is_origin(lvh) || seg_only_exclusive(first_seg(lvh)))
+				exclusive = 1;
+		}
+
 		/* Ensure mirror LV is active */
 		if (!_activate_lv(cmd, lv_mirr, exclusive)) {
 			log_error("ABORTING: Temporary mirror activation failed.");
@@ -799,7 +712,7 @@ static int _pvmove_setup_single(struct cmd_context *cmd,
 		goto out;
 
 	if (flags & PVMOVE_FIRST_TIME)
-		if (!_update_metadata(cmd, vg, lv_mirr, lvs_changed, exclusive))
+		if (!_update_metadata(lv_mirr, lvs_changed, exclusive))
 			goto_out;
 
 	/* LVs are all in status LOCKED */
